import { Component, Input, OnInit } from '@angular/core';
import { HttpClient } from '@angular/common/http';

interface Partition {
  uuid?: string; // Agregamos uuid opcional
  partitionNumber: number;
  size: number;
  type: string;
  sizeBytes: number;
  format: boolean;
  color: string;
  percentage: number;
}

@Component({
  selector: 'app-partition-assistant',
  templateUrl: './partition-assistant.component.html',
  styleUrls: ['./partition-assistant.component.css']
})
export class PartitionAssistantComponent implements OnInit {
  @Input() data: any;
  @Input() clientUuid: string | undefined;

  errorMessage = '';
  originalPartitions: any[] = [];
  disks: { diskNumber: number; totalDiskSize: number; partitions: Partition[] }[] = [];

  private apiUrl = 'http://127.0.0.1:8001/partitions';

  constructor(private http: HttpClient) {}

  ngOnInit() {
    this.initializeDisks();
  }

  initializeDisks() {
    const partitionsFromData = this.data.partitions;
    this.originalPartitions = JSON.parse(JSON.stringify(partitionsFromData));

    const disksMap = new Map<number, { totalDiskSize: number; partitions: Partition[] }>();

    partitionsFromData.forEach((partition: any) => {
      if (!disksMap.has(partition.diskNumber)) {
        disksMap.set(partition.diskNumber, { totalDiskSize: 0, partitions: [] });
      }

      const disk = disksMap.get(partition.diskNumber);
      if (partition.partitionNumber === 0) {
        disk!.totalDiskSize = this.convertBytesToGB(partition.size);
      } else {
        disk!.partitions.push({
          uuid: partition.uuid, // Incluimos el uuid
          partitionNumber: partition.partitionNumber,
          size: this.convertBytesToGB(partition.size),
          type: partition.type,
          sizeBytes: partition.size,
          format: false,
          color: '#' + Math.floor(Math.random() * 16777215).toString(16),
          percentage: 0
        });
      }
    });

    disksMap.forEach((disk, diskNumber) => {
      this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);
      this.disks.push({
        diskNumber: diskNumber,
        totalDiskSize: disk.totalDiskSize,
        partitions: disk.partitions
      });
    });
  }

  convertBytesToGB(bytes: number): number {
    return bytes / (1024 * 1024 * 1024);
  }

  updatePartitionPercentages(partitions: Partition[], totalDiskSize: number) {
    partitions.forEach((partition) => {
      partition.percentage = (partition.size / totalDiskSize) * 100;
    });
  }

  addPartition(diskNumber: number) {
    const disk = this.disks.find((d) => d.diskNumber === diskNumber);

    if (disk) {
      const remainingGB = this.getRemainingGB(disk.partitions, disk.totalDiskSize);
      if (remainingGB > 0) {
        const maxPartitionNumber =
          disk.partitions.length > 0 ? Math.max(...disk.partitions.map((p) => p.partitionNumber)) : 0;
        const newPartitionNumber = maxPartitionNumber + 1;

        disk.partitions.push({
          partitionNumber: newPartitionNumber,
          size: 0,
          type: 'NTFS',
          sizeBytes: 0,
          format: false,
          color: '#' + Math.floor(Math.random() * 16777215).toString(16),
          percentage: 0
        });
        this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);
      } else {
        this.errorMessage = 'No hay suficiente espacio libre en el disco para crear una nueva partición.';
      }
    }
  }

  updatePartitionSize(diskNumber: number, index: number, size: number) {
    const disk = this.disks.find((d) => d.diskNumber === diskNumber);
    if (disk) {
      const partition = disk.partitions[index];
      const remainingGB = this.getRemainingGB(disk.partitions, disk.totalDiskSize) + partition.size;

      if (size > remainingGB) {
        this.errorMessage = `El tamaño de la partición no puede superar el espacio libre (${remainingGB.toFixed(
          2
        )} GB).`;
      } else {
        this.errorMessage = '';
        partition.size = size;
        partition.sizeBytes = size * 1024 * 1024 * 1024;
        this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);
      }
    }
  }

  getRemainingGB(partitions: Partition[], totalDiskSize: number): number {
    const totalUsedGB = partitions.reduce((acc, partition) => acc + partition.size, 0);
    return Math.max(0, totalDiskSize - totalUsedGB);
  }

  isPartitionModified(original: any, current: Partition): boolean {
    return (
      this.convertBytesToGB(original.size) !== current.size ||
      original.type !== current.type
    );
  }

  getModifiedOrNewPartitions() {
    const modifiedPartitions: any[] = [];

    this.disks.forEach((disk) => {
      disk.partitions.forEach((partition) => {
        const originalPartition = this.originalPartitions.find(
          (p) => p.diskNumber === disk.diskNumber && p.partitionNumber === partition.partitionNumber
        );

        if (!originalPartition) {
          // Es una nueva partición
          modifiedPartitions.push({
            partition,
            diskNumber: disk.diskNumber,
            partitionNumber: partition.partitionNumber,
            isNew: true
          });
        } else if (this.isPartitionModified(originalPartition, partition)) {
          // La partición ha sido modificada
          modifiedPartitions.push({
            partition,
            diskNumber: disk.diskNumber,
            partitionNumber: partition.partitionNumber,
            isNew: false
          });
        }
      });
    });

    return modifiedPartitions;
  }

  save() {
    const modifiedPartitions = this.getModifiedOrNewPartitions();

    if (modifiedPartitions.length === 0) {
      this.errorMessage = 'No hay cambios para guardar.';
      return;
    }

    modifiedPartitions.forEach(({ partition, diskNumber, partitionNumber, isNew }) => {
      const payload = {
        diskNumber: diskNumber,
        partitionNumber: partitionNumber,
        size: partition.size,
        filesystem: partition.type,
        client: `/clients/${this.clientUuid}`
      };

      if (isNew) {
        // Es una nueva partición, usamos POST
        this.http.post(this.apiUrl, payload).subscribe(
          (response) => {
            console.log('Partición creada exitosamente:', response);
          },
          (error) => {
            console.error('Error al crear la partición:', error);
          }
        );
      } else if (partition.uuid) {
        // Es una partición existente modificada, usamos PATCH
        const patchUrl = `${this.apiUrl}/${partition.uuid}`;
        this.http.patch(patchUrl, payload).subscribe(
          (response) => {
            console.log('Partición actualizada exitosamente:', response);
          },
          (error) => {
            console.error('Error al actualizar la partición:', error);
          }
        );
      }
    });
  }

  removePartition(diskNumber: number, partition: Partition) {
    const disk = this.disks.find((d) => d.diskNumber === diskNumber);

    if (disk) {
      const index = disk.partitions.indexOf(partition);
      if (index !== -1) {
        disk.partitions.splice(index, 1);
        this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);

        if (partition.uuid) {
          // La partición existía originalmente, enviamos DELETE
          const deleteUrl = `${this.apiUrl}/${partition.uuid}`;
          this.http.delete(deleteUrl).subscribe(
            (response) => {
              console.log('Partición eliminada exitosamente:', response);
            },
            (error) => {
              console.error('Error al eliminar la partición:', error);
            }
          );
        }
      }
    }
  }
}
